varying vec3 normal;
varying vec4 pos;

void main() {
   //Material properties
   vec4 ambientMat = gl_FrontMaterial.ambient;
   vec4 diffuseMat = gl_FrontMaterial.diffuse;
   vec4 specularMat = gl_FrontMaterial.specular;
   float shininess = gl_FrontMaterial.shininess;
   
   //Light properties
   vec4 specularLight = gl_LightSource[0].specular;
   vec4 diffuseLight = gl_LightSource[0].diffuse;
   vec4 ambientLight = gl_LightSource[0].ambient;
   
   //Light equation prep
   vec4 lpos = gl_LightSource[0].position;
   vec4 ltop = normalize(lpos-pos); //light to pixel point
   vec3 lightVec = ltop.xyz;
   vec3 n = normalize(normal);
   vec3 r = -reflect(lightVec, n); //invert because reflect() expects from-light, not to-light
   r = normalize(r);
   vec3 v = -pos.xyz;
   v = normalize(v);
   //vec3 h = gl_LightSource[0].halfVector.xyz;
   //vec3 h = v - (-lightVec);
   //h = normalize(h);
    
   //Light equation
   vec4 diffuse  = diffuseMat * max(0.0, dot(n, lightVec)) * diffuseLight;
   vec4 specular = specularMat * pow(max(0.0, dot(r,v)), shininess) * specularLight;  
   //vec4 specular = specularMat * pow(max(0.0, dot(n,h)), shininess) * specularLight;  
   vec4 ambient = (ambientMat * ambientLight) + (gl_LightModel.ambient * gl_FrontMaterial.ambient);

   //Set the color
   gl_FragColor = ambient + diffuse + specular;
}