varying vec3 normal;
varying vec4 pos;
uniform sampler2D randomTex;
uniform sampler2D colorTex;
uniform sampler2D normalTex;
uniform sampler2D depthTex;
uniform sampler1D depthFunc;

float DepthToLinearZ(float dVal)
{
	float near = 1.0;
	float far = 100.0;
	//Inverse of the perpective projection
	return (-2.0*far*near) / ((dVal*(far-near))-(far+near));
}

void main()
{
   // DEPTH STUFF
   
   //useful vars
   float near = 1.0;
   float far = 100.0;
   //normalize coord
   vec2 coord = (gl_FragCoord).xy;
   coord.x = coord.x / 800.0;
   coord.y = coord.y / 600.0;
   float maxPixelDist = 10.0;
   float yOffset = (maxPixelDist/600.0) * texture2D(randomTex, coord).r; //up to maxPixelDist pixels away
   float xOffset = (maxPixelDist/800.0) * texture2D(randomTex, coord).b; //up to maxPixelDist pixels away
   
   //Get neighbor depths
   float depth   = DepthToLinearZ( texture2D(depthTex, coord).r );
   float depthT  = DepthToLinearZ( texture2D(depthTex, vec2(coord.x,coord.y+yOffset)).r );
   float depthTR = DepthToLinearZ( texture2D(depthTex, vec2(coord.x+xOffset,coord.y+yOffset)).r );
   float depthTL = DepthToLinearZ( texture2D(depthTex, vec2(coord.x-xOffset,coord.y+yOffset)).r );
   float depthR  = DepthToLinearZ( texture2D(depthTex, vec2(coord.x+xOffset,coord.y)).r );
   float depthL  = DepthToLinearZ( texture2D(depthTex, vec2(coord.x-xOffset,coord.y)).r );
   float depthB  = DepthToLinearZ( texture2D(depthTex, vec2(coord.x,coord.y-yOffset)).r );
   float depthBR = DepthToLinearZ( texture2D(depthTex, vec2(coord.x+xOffset,coord.y-yOffset)).r );
   float depthBL = DepthToLinearZ( texture2D(depthTex, vec2(coord.x-xOffset,coord.y-yOffset)).r );
   
   //Get neighbor normals
   vec4 Normal =   texture2D(normalTex, coord);
   vec4 normalT =  texture2D(normalTex, vec2(coord.x,coord.y+yOffset));
   vec4 normalTR = texture2D(normalTex, vec2(coord.x+xOffset,coord.y+yOffset));
   vec4 normalTL = texture2D(normalTex, vec2(coord.x-xOffset,coord.y+yOffset));
   vec4 normalR =  texture2D(normalTex, vec2(coord.x+xOffset,coord.y));
   vec4 normalL =  texture2D(normalTex, vec2(coord.x-xOffset,coord.y));
   vec4 normalB =  texture2D(normalTex, vec2(coord.x,coord.y-yOffset));
   vec4 normalBR = texture2D(normalTex, vec2(coord.x+xOffset,coord.y-yOffset));
   vec4 normalBL = texture2D(normalTex, vec2(coord.x-xOffset,coord.y-yOffset));
	
   //percentage of max occlusion threshold
   float maxDist = 3.0;
   float lookupT  = (depth-depthT ) / maxDist;
   float lookupTR = (depth-depthTR) / maxDist;
   float lookupTL = (depth-depthTL) / maxDist;
   float lookupR  = (depth-depthR ) / maxDist;
   float lookupL  = (depth-depthL ) / maxDist;
   float lookupB  = (depth-depthB ) / maxDist;
   float lookupBR = (depth-depthBR) / maxDist;
   float lookupBL = (depth-depthBL) / maxDist;
	
	//Ambient occlusion terms
   float amContT  = (1.0-(texture1D(depthFunc,lookupT).r));
   float amContTR = (1.0-(texture1D(depthFunc,lookupTR).r));
   float amContTL = (1.0-(texture1D(depthFunc,lookupTL).r));
   float amContR  = (1.0-(texture1D(depthFunc,lookupR).r));
   float amContL  = (1.0-(texture1D(depthFunc,lookupL).r));
   float amContB  = (1.0-(texture1D(depthFunc,lookupB).r));
   float amContBR = (1.0-(texture1D(depthFunc,lookupBR).r));
   float amContBL = (1.0-(texture1D(depthFunc,lookupBL).r));
   
	float lengthThresh = 0.1;
   amContT  = (length(Normal-normalT)   > lengthThresh) ? amContT : 1.0;
   amContTR = (length(Normal-normalTR)  > lengthThresh) ? amContTR : 1.0;
   amContTL = (length(Normal-normalTL)  > lengthThresh) ? amContTL : 1.0;
   amContR  = (length(Normal-normalR)   > lengthThresh) ? amContR : 1.0;
   amContL  = (length(Normal-normalL)   > lengthThresh) ? amContL : 1.0;
   amContB  = (length(Normal-normalB)   > lengthThresh) ? amContB : 1.0;
   amContBR = (length(Normal-normalBR)  > lengthThresh) ? amContBR : 1.0;
   amContBL = (length(Normal-normalBL)  > lengthThresh) ? amContBL : 1.0;
   
   //What should the final term be? Min of all of them
   float ambientContribution = (amContT + amContTR + amContTL + amContR + amContL + amContB + amContBR + amContBL) / 8.0;
   /*float ambientContribution = amContT;
   if(amContTR < ambientContribution) {
      ambientContribution = amContTR;
   }
   if(amContTL < ambientContribution) {
      ambientContribution = amContTL;
   }
   if(amContR < ambientContribution) {
      ambientContribution = amContR;
   }
   if(amContL < ambientContribution) {
      ambientContribution = amContL;
   }
   if(amContB < ambientContribution) {
      ambientContribution = amContB;
   }
   if(amContBR < ambientContribution) {
      ambientContribution = amContBR;
   }
   if(amContBL < ambientContribution) {
      ambientContribution = amContBL;
   }*/
	
	
	// REGULAR AMBIENT ONLY SHADER
	
	//Material properties
   vec4 ambientMat = gl_FrontMaterial.ambient;
   
   //Light properties
   vec4 ambientLight = gl_LightSource[0].ambient;
   
   //Light equation
   vec4 ambient = (ambientMat * ambientLight) + (gl_LightModel.ambient * gl_FrontMaterial.ambient);

   //Set the color
   gl_FragColor = (ambient*ambientContribution);
}
